#ifndef __GUI_H_
#define __GUI_H_

#include "graphics.h"
#include "gameviewer.h"
#include <string>
#include <vector>
#include <list>

enum GuiEventType
{
	GUIEVENT_NOTHING,
	GUIEVENT_BUTTONCLICK,
	GUIEVENT_IMAGECLICK,
	GUIEVENT_MAPVIEWER_BLOCKCLICK,
	GUIEVENT_MAPVIEWER_BLOCKHOVER,
	//GUIEVENT_MAPVIEWER_BLOCKSEL_CTRL,
	//GUIEVENT_MAPVIEWER_BLOCKSEL_SHIFT,
	GUIEVENT_REQUEST_HIDEMOUSE,
	GUIEVENT_REQUEST_SHOWMOUSE,
	GUIEVENT_REQUEST_WARPMOUSE,
	GUIEVENT_ASKLINE_RETURN,
	GUIEVENT_ASKLINE_CANCEL,
	GUIEVENT_REQUEST_DRAWBACKGROUND
};

struct GuiEvent
{
	GuiEvent();
	GuiEvent(std::string cname, GuiEventType type_);
	void AddName(std::string cname);
	GuiEventType type;
	std::vector<std::string> cnames;
	int x, y;
	std::string text;
};

struct ImageWithName
{
	SDL_Surface *s;
	std::string n;
};

class GuiKey
{
public:
	GuiKey();
	GuiKey(SDLKey sdlk_, bool shiftstate_);
	void SetSDLK(SDLKey sdlk_, bool shiftstate_);
	SDLKey GetSDLK();
	char GetChar();
private:
	SDLKey sdlk;
	bool shiftstate;
};

class GuiComponent
{
public:
	virtual ~GuiComponent();
	GuiComponent(int w, int h, std::string name, Fonts fonts);

	virtual void SetBasePlace(int x, int y);
	virtual void SetRelPlace(int x, int y);
	void SetSize(int w, int h);
	void GetRelPlace(int &x, int &y);
	void GetSize(int &w, int &h);
	void SetName(std::string name);
	std::string GetName();
	//TTF_Font *GetDefaultFont();
	void SetProperties(int w, int h, std::string name, Fonts fonts);
	virtual void SetSurface(SDL_Surface *surface);
	SDL_Surface* GetSurface();

	void Delete();
	bool Deleted();
	
	void SetDisabled(bool disabled_);
	void Disable();
	void Enable();
	bool IsDisabled();
	
	virtual bool GetEvent(GuiEvent *event);

	virtual void Draw(int x, int y, int w, int h, bool is_selected);
	//return true if did something with the info (at least if click was on component's area)
	virtual bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
	//return true if did something with the info
	virtual bool Keyboard(GuiKey key);
	//return true if did something with the info
	virtual bool SDLK(SDLKey key, bool down);

protected:
	int cw, ch;
	int basex, basey;
	int rel_x, rel_y;
	int screen_x, screen_y;
	std::string cname;
	SDL_Surface *surface;
	Fonts fonts;
	std::list<GuiEvent> events;
private:
	bool delete_me, disabled;
};

class GuiContainer : public GuiComponent
{
public:
	virtual ~GuiContainer();
	GuiContainer(int w, int h, std::string name, std::string title,
			Uint8 bg_r, Uint8 bg_g, Uint8 bg_b, Fonts fonts);
	void SetBasePlace(int x, int y);
	void SetRelPlace(int x, int y);
	void SetSurface(SDL_Surface *s);
	void AddComponent(GuiComponent*, int x, int y);
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
	bool Keyboard(GuiKey key);
	bool SDLK(SDLKey key, bool down);
	bool GetEvent(GuiEvent *event);
protected:
	void SetSize(int x, int y);
	std::vector<GuiComponent*> components;
	int selected_component;
	Uint8 bgcolor_r, bgcolor_g, bgcolor_b;
	bool bg_drawn;
	std::string title;
	int titleh;
	bool titlegrabbed;
};

class GuiLabel : public GuiComponent
{
public:
	virtual ~GuiLabel();
	GuiLabel(std::string name, std::string text, Fonts fonts);
	void SetText(std::string text);
	void Draw(int x, int y, int w, int h, bool is_selected);
private:
	std::string text;
};

class GuiMultilineLabel : public GuiComponent
{
public:
	virtual ~GuiMultilineLabel();
	GuiMultilineLabel(std::string name, ViewableText &vtext, Fonts fonts);
	void Draw(int x, int y, int w, int h, bool is_selected);
	void SetText(ViewableText &vtext);
private:
	ViewableText vtext;
};

class GuiTextView : public GuiComponent
{
public:
	virtual ~GuiTextView();
	GuiTextView(int w, int h, std::string name, ViewableText *vtext, Fonts fonts);
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
private:
	ViewableText *vtext;
	int text_scrollpos_x, text_scrollpos_y;
	int text_oldpos_x, text_oldpos_y;
	bool pressed, pressed_on_v_scrollbar, pressed_on_h_scrollbar;
	int dragstart_x, dragstart_y;
};

class GuiTextButton : public GuiComponent
{
public:
	virtual ~GuiTextButton();
	GuiTextButton(int w, int h, std::string name, std::string label, Fonts fonts);
	void SetLabel(std::string text);
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
private:
	bool pressed;
	std::string label;
};

class GuiImage : public GuiComponent
{
public:
	virtual ~GuiImage();
	GuiImage(std::string name, SDL_Surface *image, Fonts fonts);
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
private:
	bool pressed;
	SDL_Surface *image;
};

enum GuiScrollbarType
{
	GUISCROLLBAR_HORIZ,
	GUISCROLLBAR_VERT
};

class GuiScrollbar : public GuiComponent
{
public:
	virtual ~GuiScrollbar();
	GuiScrollbar(int w, int h, std::string name, GuiScrollbarType type,
			int range, int barlength, int position, Fonts fonts);
	void SetProperties(int range, int barlength, int position);
	void SetRange(int range);
	void SetBarLength(int barlength);
	void SetPosition(int position);
	int GetPosition();
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
protected:
	void CalcBar(int &x, int &y, int &w, int &h, double &pos_px_per_real_px);
	GuiScrollbarType type;
	int range;
	int barlength;
	int position;
	bool pressed;
	int startposition;
	int mstartx, mstarty;
};

class GuiScrollable : public GuiComponent
{
public:
	virtual ~GuiScrollable();
	GuiScrollable(int w, int h, std::string name, GuiComponent *c, Fonts f);
	GuiScrollable(int w, int h, std::string name, Fonts f);
	void SetBasePlace(int x, int y);
	void SetRelPlace(int x, int y);
	void SetSurface(SDL_Surface *s);
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
	bool GetEvent(GuiEvent *event);
	void SetComponent(GuiComponent *c);
	GuiComponent* GetComponent();
protected:
	SDL_Surface *drawsurface;
	GuiComponent *component;
	GuiScrollbar *scrollbar_vert, *scrollbar_horiz;
	int position_x, position_y;
	int com_w, com_h;
	bool w_fits, h_fits;
};

class GuiImageList : public GuiScrollable
{
public:
	virtual ~GuiImageList();
	GuiImageList(int w, int h, std::string name, Fonts fonts, std::vector<ImageWithName> images);
protected:
};

class GuiGameViewer : public GuiComponent
{
public:
	GuiGameViewer(int w, int h, std::string name, int whattodraw, Map *m,
			ObjectImageLib *objectimagelib, TileLib *tilelib,
			MapImageLib *mapimagelib, Fonts fonts);
	
	void Draw(int x, int y, int w, int h, bool is_selected);
	bool Mouse(int mx, int my, int movex, int movey, bool lbutton, bool rbutton);
	bool SDLK(SDLKey key, bool down);
	
	void SetMap(Map *m);

	void SetMapOffset(int x, int y);
	void AddMapOffset(int xchange, int ychange);

	int GetWhatToDraw();
	void SetWhatToDraw(int whattodraw);

	void SetFloorToDraw(int floortodraw_);

	int GetMouseSettings();
	void SetMouseSettings(int mousesettings);

	void RollAnims(int timenow_ms);

private:
	bool GetBlockBaseInPxPlace(int on_map_x, int on_map_y, int &bx, int &by);

	Map *map;
	ObjectImageLib *objectimagelib;
	TileLib *tilelib;
	MapImageLib *mapimagelib;

	int view_map_x_offset, view_map_y_offset;

	int whattodraw;
	int floortodraw;
	int mousesettings;

	bool rpressed;
	int rstartx, rstarty;

	bool lpressed;
	int lmapstartx, lmapstarty;

	bool lshift, rshift, lctrl, rctrl;
	bool ctrl_on_hilighted;

	int lastbx, lastby;
	int lasthoverbx, lasthoverby;
};


///////////

class GuiMessageBox : public GuiContainer
{
public:
	virtual ~GuiMessageBox();
	GuiMessageBox(std::string name, std::string title, std::string message, Fonts fonts);
	bool Keyboard(GuiKey key);
	bool GetEvent(GuiEvent *event);
private:
};

class GuiAskLine : public GuiContainer
{
public:
	virtual ~GuiAskLine();
	GuiAskLine(std::string name, std::string title, std::string message, Fonts fonts);
	void SetLine(std::string text);
	bool Keyboard(GuiKey key);
	bool GetEvent(GuiEvent *event);
private:
	std::string line;
	GuiLabel *linelabel;
};

///////////////

class Gui
{
public:
	Gui(SDL_Surface *surface, Fonts *fonts, Uint8 bg_r, Uint8 bg_g, Uint8 bg_b);
	~Gui();
	void AddComponent(GuiComponent*, int x, int y);
	bool Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton);
	bool Keyboard(GuiKey k);
	bool SDLK(SDLKey key, bool down);
	bool GetEvent(GuiEvent *event);
	void Draw(int x, int y, int w, int h);

	void StartMessageBox(std::string name, std::string title, std::string message);
	void StartAskIntValue(std::string name, std::string title, std::string message);
	//void StartImageDialog(std::string name, std::vector<ImageWithName> images);
private:
	GuiContainer *basecontainer;
	Fonts *fonts;
};

#endif

